unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ZylSerialPort, ExtCtrls, Menus;

type
  TMainForm = class(TForm)
    ZylSerialPort: TZylSerialPort;
    Memo: TMemo;
    btnOpen: TButton;
    btnSendStr: TButton;
    btnClose: TButton;
    lstPorts: TListBox;
    rgMode: TRadioGroup;
    lstBaudRate: TListBox;
    lblPort: TLabel;
    lblBaudRate: TLabel;
    rbDataWidth: TRadioGroup;
    rbParity: TRadioGroup;
    rbStopBits: TRadioGroup;
    rbHwFlowControl: TRadioGroup;
    rbSwFlowControl: TRadioGroup;
    btnApply: TButton;
    gbEnable: TGroupBox;
    cbDTR: TCheckBox;
    cbRTS: TCheckBox;
    poplstPorts: TPopupMenu;
    miRefreshPorts1: TMenuItem;
    efSentStr: TMemo;
    cbShowHiddenChars: TCheckBox;
    procedure btnOpenClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ZylSerialPortLineStatusChange(Sender: TObject;
      LineStatus: TLineStatusSet);
    procedure btnSendStrClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ZylSerialPortDisconnect(Sender: TObject; Port: TCommPort);
    procedure ZylSerialPortConnect(Sender: TObject; Port: TCommPort);
    procedure ZylSerialPortReceive(Sender: TObject; Buffer: AnsiString);
    procedure ZylSerialPortSend(Sender: TObject; Buffer: AnsiString);
    procedure btnApplyClick(Sender: TObject);
    procedure miRefreshPorts1Click(Sender: TObject);
    procedure ZylSerialPortFault(Sender: TObject; Port: TCommPort);
  private
    { Private declarations }
    procedure ApplySettings;
    procedure GetSerialPorts();
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.btnOpenClick(Sender: TObject);
begin
 if lstPorts.ItemIndex < 0 then
  begin
    ShowMessage('Select a port!');
    Exit;
  end;
  ZylSerialPort.Port := ZylSerialPort.StringToCommPort(lstPorts.Items[lstPorts.ItemIndex]);
  if ZylSerialPort.Port = spCustom then
    ZylSerialPort.CustomPortName := lstPorts.Items[lstPorts.ItemIndex];
  ApplySettings;
  ZylSerialPort.NeedSynchronization := True;
  ZylSerialPort.AutoReconnect := True;
  ZylSerialPort.Open;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ZylSerialPort.Close();
end;

procedure TMainForm.ZylSerialPortLineStatusChange(Sender: TObject;
  LineStatus: TLineStatusSet);
var
  strLine: AnsiString;
begin
  strLine := '';
  if lsCTS in LineStatus then
    strLine := strLine + 'CTS ';
  if lsDSR in LineStatus then
    strLine := strLine + 'DSR ';
  if lsRING in LineStatus then
    strLine := strLine + 'RING ';
  if lsDCD in LineStatus then
    strLine := strLine + 'DCD ';
  Memo.Lines.Add('Line Change: ' + strLine);
end;

procedure TMainForm.btnSendStrClick(Sender: TObject);
begin
  ZylSerialPort.SendString(efSentStr.Text + #13#10);
end;

procedure TMainForm.btnCloseClick(Sender: TObject);
begin
  ZylSerialPort.Close;
end;

procedure TMainForm.GetSerialPorts();
//var
  //comPorts: TCommPortSet;
  //i: TCommPort;
  //nPort: Integer;
begin
  lstPorts.Items.Clear;
  ZylSerialPort.GetExistingCommPortNames(lstPorts.Items);
  {comPorts := ZylSerialPort.GetExistingCommPorts();
  for i := spCOM1 to spCOM254 do
  begin
    if i in comPorts then
      lstPorts.Items.Add(ZylSerialPort.CommPortToString(i));
  end;    }
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  GetSerialPorts();
  lstBaudRate.ItemIndex := lstBaudRate.Items.IndexOf('4800');
end;

procedure TMainForm.ZylSerialPortDisconnect(Sender: TObject; Port: TCommPort);
begin
  Memo.Lines.Add('Disconnected from ' + ZylSerialPort.CommPortToString(Port));
end;

procedure TMainForm.ZylSerialPortFault(Sender: TObject; Port: TCommPort);
begin
  Memo.Lines.Add('Faulted: ' + ZylSerialPort.CommPortToString(Port));
end;

procedure TMainForm.ZylSerialPortConnect(Sender: TObject; Port: TCommPort);
begin
  Memo.Lines.Add('Connected to ' + ZylSerialPort.CommPortToString(Port));
end;

procedure TMainForm.ZylSerialPortReceive(Sender: TObject; Buffer: AnsiString);
var
  strBuffer: AnsiString;
  i: Integer;
begin
  if rgMode.ItemIndex = 0 then
  begin
    if cbShowHiddenChars.Checked then
      Memo.Lines.Add(ZylSerialPort.ReplaceControlChars(Buffer))
    else
      Memo.Lines.Add(Buffer);  
  end
  else if rgMode.ItemIndex = 1 then
  begin
    strBuffer := '';
    for i := 1 to Length(Buffer) do
      strBuffer := strBuffer + IntToStr(Ord(Buffer[i]));
    Memo.Lines.Add(strBuffer)
  end
  else
  begin
    strBuffer := '';
    for i := 1 to Length(Buffer) do
      strBuffer := strBuffer + IntToHex(Ord(Buffer[i]), 1);
    Memo.Lines.Add(strBuffer)
  end;
end;

procedure TMainForm.ZylSerialPortSend(Sender: TObject; Buffer: AnsiString);
begin
  Memo.Lines.Add('Sent: ' + ZylSerialPort.ReplaceControlChars(Buffer));
end;

procedure TMainForm.ApplySettings;
begin
  ZylSerialPort.BaudRate := ZylSerialPort.IntToBaudRate(StrToInt(lstBaudRate.Items[lstBaudRate.ItemIndex]));
  ZylSerialPort.DataWidth := TDataWidth(rbDataWidth.ItemIndex);
  ZylSerialPort.Parity := TParityBits(rbParity.ItemIndex);
  ZylSerialPort.StopBits := TStopBits(rbStopBits.ItemIndex);
  ZylSerialPort.HwFlowControl := THwFlowControl(rbHwFlowControl.ItemIndex);
  ZylSerialPort.SwFlowControl := TSwFlowControl(rbSwFlowControl.ItemIndex);
  if cbDTR.Checked then
    ZylSerialPort.EnableDTROnOpen := True
  else
    ZylSerialPort.EnableDTROnOpen := False;

  if cbRTS.Checked then
    ZylSerialPort.EnableRTSOnOpen := True
  else
    ZylSerialPort.EnableRTSOnOpen := False;
end;

procedure TMainForm.btnApplyClick(Sender: TObject);
begin
  ApplySettings;
end;

procedure TMainForm.miRefreshPorts1Click(Sender: TObject);
begin
  lstPorts.Clear;
  GetSerialPorts;
end;

end.
